# -*- coding:utf-8 -*-
import urllib
import base64
import hmac
import hashlib
import requests
import wave
import json
import time

class tts_tencent_data_authorization_class:
    '''
    腾讯语音合成鉴权数据
    '''
    def __init__(self):
        #用户鉴权参数
        self.AppId = 1252113161
        self.SecretId = 'AKIDTp12qQmwufBSWN2SfgU3OYUDhvxkIaeE'
        self.SecretKey = 'NSHtIVYTHm48yZ7RWN3UiiRyFKBpFRO8'
        self.Expired = 3600

    def verify_param(self):
        if len(str(self.AppId)) == 0:
             print('AppId can not empty')
        if len(str(self.SecretId)) == 0:
             print('SecretId can not empty')
        if len(str(self.SecretKey)) == 0:
             print('SecretKey can not empty')

    def init_auth(self, appid, secret_id, secret_key):
        self.AppId = appid
        self.SecretId = secret_id
        self.SecretKey = secret_key

    def generate_sign(self, request_data):
        url = "tts.cloud.tencent.com/stream"
        sign_str = "POST" + url + "?"
        sort_dict = sorted(request_data.keys())
        for key in sort_dict:
            sign_str = sign_str + key + "=" + urllib.parse.unquote(str(request_data[key])) + '&'
        sign_str = sign_str[:-1]
        sign_bytes = sign_str.encode('utf-8')
        key_bytes = self.SecretKey.encode('utf-8')
        print(sign_bytes)
        tts_tencent_data_authorization_class = base64.b64encode(hmac.new(key_bytes, sign_bytes, hashlib.sha1).digest())
        return tts_tencent_data_authorization_class.decode('utf-8')

class tts_tencent_data_request_class:
    '''
    腾讯语音合成请求数据
    '''
    def __init__(self, save=None, text=None, sessionId=None):
        self.Text = text if text else '要被转化成语音的文字，最大支持500字符。'
        self.Action = 'TextToStreamAudio' # 请求参数固定值，不能改
        self.Codec = 'pcm'          #返回音频格式：Python SDK只支持pcm格式  #pcm：返回二进制 pcm 音频，使用简单，但数据量大。
        self.Expired = 3600         #鉴权有效时间 单位 s,不设置默认为一小时
        self.ModelType = 1          #模型类型，1：默认模型
        self.PrimaryLanguage = 1    #主语言类型 1：中文（默认）2：英文
        self.ProjectId = 0          #项目 ID，用户自定义，默认为0。
        self.SampleRate = 16000     #音频采样率：16000：16k（默认） 8000：8k
        self.SessionId = str(sessionId if sessionId else int(time.time()))          #一次请求对应一个 SessionId，会原样返回，建议传入类似于 uuid 的字符串防止重复。
        self.Speed = 1              #语速，范围：[-2，2]，分别对应不同语速：［0.6倍,0.8倍,1.0倍（默认）,1.2倍,1.5倍]
        self.Volume = 5             #音量大小，范围：[0，10]，分别对应11个等级的音量，默认值为0，代表正常音量。没有静音选项。
        self.VoiceType = 0          #音色： #0：亲和女声（默认） #1：亲和男声 #2：成熟男声 #3：活力男声 #4：温暖女声 #5：情感女声 #6：情感男声 #标准音色 #10510000-智逍遥，阅读男声 #1001-智瑜，情感女声 #1002-智聆，通用女声 #1003-智美，客服女声 #1004-智云，通用男声 #1005-智莉，通用女声 #1007-智娜，客服女声 #1008-智琪，客服女声 #1009-智芸，知性女声 #1010-智华，通用男声 #1017-智蓉，情感女声 #1018-智靖，情感男声 #1050-WeJack，英文男声 #1051-WeRose，英文女声 #精品音色 #精品音色拟真度更高，价格不同于标准音色，查看购买指南 #100510000-智逍遥，阅读男声 #101001-智瑜，情感女声 #101002-智聆，通用女声 #101003-智美，客服女声 #101004-智云，通用男声 #101005-智莉，通用女声 #101006-智言，助手女声 #101007-智娜，客服女声 #101008-智琪，客服女声 #101009-智芸，知性女声 #101010-智华，通用男声 #101011-智燕，新闻女声 #101012-智丹，新闻女声 #101013-智辉，新闻男声 #101014-智宁，新闻男声 #101015-智萌，男童声 #101016-智甜，女童声 #101017-智蓉，情感女声 #101018-智靖，情感男声 #101019-智彤，粤语女声 #101020-智刚，新闻男声 #101021-智瑞，新闻男声 #101022-智虹，新闻女声 #101023-智萱，聊天女声 #101024-智皓，聊天男声 #101025-智薇，聊天女声 #101026-智希，通用女声 #101027-智梅，通用女声 #101028-智洁，通用女声 #101029-智凯，通用男声 #101030-智柯，通用男声 #101031-智奎，通用男声 #101032-智芳，通用女声 #101033-智蓓，客服女声 #101034-智莲，通用女声 #101035-智依，通用女声 #101040-智川，四川女声 #101050-WeJack，英文男声 #101051-WeRose，英文女声
        self.SavePath = save if save else 'tts_tencent.wav'        #文件存放位置

    def verify_param(self):
        if len(str(self.Action)) == 0:
             print('Action can not empty')
        if len(str(self.SampleRate)) == 0:
             print('SampleRate is not set, assignment default value 16000')
             self.SampleRate = 16000

    def init_param(self, save, text, action=None, codec=None, expired=None, model_type=None, prim_lan=None, project_id=None, sample_rate=None, session_id=None, speed=None, voice_type=None, volume=None):
        self.Action = action if action else self.Action
        self.Text = text
        self.Codec = codec if codec else self.Codec
        self.Expired = expired if expired else self.Expired
        self.ModelType = model_type if model_type else self.ModelType
        self.PrimaryLanguage = prim_lan if prim_lan else self.PrimaryLanguage
        self.ProjectId = project_id if project_id else self.ProjectId
        self.SampleRate = sample_rate if sample_rate else self.SampleRate
        self.SessionId = session_id if session_id else self.SessionId
        self.Speed = speed if speed else self.Speed
        self.VoiceType = voice_type if voice_type else self.VoiceType
        self.Volume = volume if volume else self.Volume
        self.SavePath = save

class tts_tencent_class():
    def __init__(self, auth:tts_tencent_data_authorization_class=None):
        self.auth = auth if auth else tts_tencent_data_authorization_class()

    def tts_tencent_req_by_text(self, save, text):
        req = tts_tencent_data_request_class()
        req.init_param(save, text)
        self.tts_tencent_req(req)

    def tts_tencent_req(self, req:tts_tencent_data_request_class=None):
        request_data = dict()
        request_data['Action'] = 'TextToStreamAudio'
        request_data['AppId'] = self.auth.AppId
        request_data['Codec'] = req.Codec
        request_data['Expired'] = int(time.time()) + self.auth.Expired
        request_data['ModelType'] = req.ModelType
        request_data['PrimaryLanguage'] = req.PrimaryLanguage
        request_data['ProjectId'] = req.ProjectId
        request_data['SampleRate'] = req.SampleRate
        request_data['SecretId'] = self.auth.SecretId
        request_data['SessionId'] = req.SessionId
        request_data['Speed'] = req.Speed
        request_data['Text'] = req.Text
        request_data['Timestamp'] = int(time.time())
        request_data['VoiceType'] = req.VoiceType
        request_data['Volume'] = req.Volume

        signature = self.auth.generate_sign(request_data = request_data)
        header = {
            "Content-Type": "application/json",
            "Authorization": signature
        }
        url = "https://tts.cloud.tencent.com/stream"

        r = requests.post(url, headers=header, data=json.dumps(request_data), stream = True)
        '''
        if str(r.content).find("Error") != -1 :
            print(r.content)
            return
        '''
        i = 1
        wavfile = wave.open(req.SavePath, 'wb')
        wavfile.setparams((1, 2, 16000, 0, 'NONE', 'NONE'))
        for chunk in r.iter_content(1000):
            if (i == 1) & (str(chunk).find("Error") != -1) :
                print(chunk)
                return False
            i = i + 1
            wavfile.writeframes(chunk)
        wavfile.close()
        return req


SUBMIT_GIT = False
NAME = 'zqpy'
DESCRIPTION = '使用老版本:pip install zqpy==0.1.48 \n 使用新版本:pip install zqpy \n 新老版本区别在于：最前面位 0（老版本）1（新版本）'
URL = 'https://gitee.com/qingBB/zqpy'
EMAIL = '1620829248@qq.com'
AUTHOR = '周清'
REQUIRES_PYTHON = '>=3'
# use in __version__.py
VERSION = False
# use in requirements.txt
REQUIRED = False 
# opertion require package


import os, sys
from setuptools import find_packages, setup, Command
import setup_file as zq_file

here_curr = os.path.abspath(os.path.dirname(__file__))

try:
    with open(os.path.join(here_curr, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

# VERSION 每次提交自动升级，所以不需要再手动改，除非是大版本提交
if not VERSION:
    project_slug = NAME.lower().replace("-", "_").replace(" ", "_")
    with open(os.path.join(here_curr, project_slug, '__version__.py')) as f:
        about = {}
        exec(f.read(), about)
        VERSION = about['__version__']

if not REQUIRED:
    content = zq_file.file_read('%s/%s/requirements.txt'%(here_curr, NAME))
    content = content.replace('~pencv_python', 'opencv_python')
    zq_file.file_write('%s/%s/requirements.txt'%(here_curr,NAME), content)
    REQUIRED = content.splitlines()

class UploadCommand(Command):
    """setup.py  upload"""

    user_options = []
    @staticmethod
    def status(s):
        """print"""
        print('\033[1m{0}\033[0m'.format(s))
    

    def initialize_options(self):
        print('初始化选项')

    def finalize_options(self):
        print('最终选项')

    def run(self):
        print('开始运行')

        self.status('build source wheel(general) distribution')
        os.system('%s setup.py sdist bdist_wheel --universal'%(sys.executable))

        self.status('pass twine upload PyPI')
        os.system('twine upload --repository-url https://upload.pypi.org/legacy/ dist/*')

        print(sys.executable, os.getcwd(), os.path.abspath('./build/lib'))
        for item in sys.path:
            if item.endswith("\\lib\\site-packages"):
                zq_file.dir_copy("./build/lib", item)
                print('./build/lib -> 复制到 %s\n\n'%item)

        self.status('push to git')
        if SUBMIT_GIT:
            os.system('git tag v{0}'.format(VERSION))
            os.system('git push --tags')
            os.system('git add .')
            os.system('git stage')
            os.system('git commit -m "{0}{1}"'.format(VERSION, input('请输入提交日志信息-> ')))
            os.system('git push')

        os.chdir(here_curr)
        # 自动升级下一个小版本
        versionPath = os.path.join(here_curr, NAME, '__version__.py')
        strValue = zq_file.file_read(versionPath)
        versionList = VERSION.split('.')
        strValue = strValue.replace(versionList[2],str(int(versionList[2])+1), -1)
        zq_file.file_write(versionPath, strValue)
        print('自动版本号+1  完成\n\n')

        Clear()
        # sys.exit()
        input()

def Clear():
    for item in ["build", "dist", "__pycache__", "%s.egg-info"%NAME]:
        zq_file.dir_remove('%s/%s'%(here_curr, item), True)
Clear()
setup(
    name=NAME,                      #包名称
    version=VERSION,                #包版本
    author=AUTHOR,                  #程序的作者
    author_email=EMAIL,             #程序的作者的邮箱地址
    maintainer=AUTHOR,              #维护者
    maintainer_email=EMAIL,         #维护者的邮箱地址
    url=URL,                        #程序的官网地址
    license='MIT',                  #程序的授权信息
    description=DESCRIPTION,        #程序的简单描述
    long_description=long_description,  #程序的详细描述
    long_description_content_type='text/markdown', #程序的详细描述类型
    # platforms=,                   #程序适用的软件平台列表
    classifiers=[                   #程序的所属分类列表
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    # keywords=,                    #程序的关键字列表
    packages=find_packages(exclude=["tests", "*.tests", "*.tests.*", "tests.*"]),   #需要处理的包目录(通常为包含 __init__.py 的文件夹)
    py_modules=[],                  #需要打包的 Python 单文件列表
    # download_url=,                #程序的下载地址
    cmdclass={                      #添加自定义命令
        'upload': UploadCommand,
    },
    # package_data=[],                   #指定包内需要包含的数据文件
    include_package_data=True,      #自动包含包内所有受版本控制(cvs/svn/git)的数据文件
    # exclude_package_data=[],        #当 include_package_data 为 True 时该选项用于排除部分文件
    # data_file=[],                   #打包时需要打包的数据文件，如图片，配置文件等
    ext_modules=[],                 #指定扩展模块
    scripts=[],                     #指定可执行脚本,安装时脚本会被安装到系统 PATH 路径下
    package_dir=[],                 #指定哪些目录下的文件被映射到哪个源码包
    # entry_points={                #动态发现服务和插件，下面详细讲
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },    
    python_requires=REQUIRES_PYTHON,    #指定运行时需要的Python版本
    requires=[],                    #指定依赖的其他包
    provides=[],                    #指定可以为哪些模块提供依赖
    install_requires=REQUIRED,      #安装时需要安装的依赖包
    extras_require={                #当前包的高级/额外特性需要依赖的分发包
        # 'fancy feature': ['django'],
    },          
    tests_require=[],               #在测试时需要使用的依赖包
    setup_requires=[],              #指定运行 setup.py 文件本身所依赖的包
    # dependency_links='',            #指定依赖包的下载地址
    zip_safe=False,                  #不压缩包，而是以目录的形式安装
)



import os, sys, time
import zq_file, zq_const

log_save = False
log_print_event = []
def log_print_record(fun):
    ''' 日志记录到文档 '''
    def wrapper(*args, **kwargs):
        if log_save:
            #将打印临时保存
            log_path = zq_const.const_merage_logs_dir('save_logs', time.strftime('%Y-%m-%d'))
            temp_log_path = zq_const.const_merage_logs_dir('tlog.txt')
            sys.stdout = open(temp_log_path, 'w', encoding='utf-8')
            fun(*args, **kwargs)
            sys.stdout.close()
            with open(temp_log_path, 'r', encoding='utf-8')as tmp:
                with open(log_path, 'a+', encoding='utf-8')as fp:
                    #写入日志的格式
                    fp.write('[{}][{}]:{}'.format(time.strftime('%Y-%m-%d %X'), fun.__name__,tmp.read()))
        else:
            fun(*args, **kwargs)
    return wrapper

@log_print_record
def log_print(*args, **kwargs):
    ''' 日志打印 '''
    if log_print_event and len(log_print_event) > 0:
        for print_item in log_print_event:
            if print_item:
                print_item(*args, **kwargs)
            else:
                log_print_remove_event(print_item)
    if len(args) > 0 and ('debug' in args[0] or 'warning' in args[0] or 'error' in args[0]):
        print(*args, **kwargs)
    else:
        print('notag', *args, **kwargs)

def log_debug(*args, **kwargs):
    ''' 一般日志打印 '''
    args = tuple(['[%s]debug: '%time.strftime('%Y-%m-%d %H:%M:%S')] + list(args))
    log_print(*args, **kwargs)
    
def log_warning(*args, **kwargs):
    ''' 警告日志打印 '''
    args = tuple(['[%s]warning: '%time.strftime('%Y-%m-%d %H:%M:%S')] + list(args))
    log_print(*args, **kwargs)
    
def log_error(*args, **kwargs):
    ''' 错误日志打印 '''
    args = tuple(['[%s]error: '%time.strftime('%Y-%m-%d %H:%M:%S')] + list(args))
    log_print(*args, **kwargs)
    

def log_print_append_event(fun):
    ''' 注册日志相应事件 '''
    log_print_event.append(fun)

def log_print_remove_event(fun):
    ''' 销毁日志相应事件 '''
    log_print_event.remove(fun)

def log_progress_bar(portion, total, desc = ''):
    '''
    打印进度条
    :param portion: 已经接收的数据量
    :param total: 总数据量
    :return: 接收数据完成，返回True
    '''
    part = total / 50  # 1%数据的大小
    count = int(portion / part)
    sys.stdout.write('\r')
    sys.stdout.write('debug: %s'%desc+('[%-50s]%.2f%%' % (('>' * count), portion / total * 100)))
    sys.stdout.flush()

    if portion >= total:
        sys.stdout.write('\n')
        return True
    return False

def log_take_up_time(fun):
    ''' 日志记录耗时 '''
    def wrapper(*args, **kwargs):
        start = time.time()
        result = fun(*args, **kwargs)
        end = time.time()
        log_debug('运行耗时 %s : %s'%(fun.__qualname__, end-start))
        return result
    return wrapper


# import zq_tts_pyttsx3 as tts
# tts.tts_pyttsx3_speek('66666')

# import zq_tts_win32com as tts
# tts.tts_win32com_speek('66666')

# import zq_tts_tencent as tts
# tts.tts_tencent_class().tts_tencent_req(tts.tts_tencent_data_request_class(save='./tencent_tts.wav', text='9999哈哈哈'))

# import zq_tts_youdao as tts
# tts.tts_youdao_web_class().tts_youdao_req(save='./youdao_tts.wav', text='9999哈哈哈')


import asyncio
import websockets
import websockets_routes
import threading
import zq_log

''' 示例 https://www.jianshu.com/p/b7df44b7c7f7
#region server
async def echo(websocket, path):
    #fetch msg
    async for message in websocket:
        zq_log.log_debug('%s get message:%s'%(path, message))
        await websocket.send('server recv: %s'%message)

async def main():
    # start a websocket server
    async with websockets.serve(echo, 'localhost', 8800):
        await asyncio.Future()  # run forever

asyncio.run(main())
#endregion

#region client
async def hello():
    async with websockets.connect('ws://localhost:8800') as websocket:
        await websocket.send('Hello world!')
        await websocket.recv()

asyncio.run(hello())
#endregion
'''

#region 服务器
# 初始化一个router对象，路由装饰器
ws_server_router = websockets_routes.Router()

async def ws_server_main(ip='localhost', port=8800):
    async with websockets.serve(lambda x, y: ws_server_router(x, y), ip, port):
        zq_log.log_debug('ws:', ip, port)
        await asyncio.Future()  # run forever

@ws_server_router.route('/v1/test/{status}') #添加router的route装饰器，它会路由uri。
async def test_status(websocket, path):
    async for message in websocket:
        zq_log.log_debug('server recv message:%s,%s,%s'%(path, path.params['status'], message))
        if (path.params['status'] == 'ok'):
            await  websocket.send('server recv status:%s message:%s'%(path.params['status'], message))
        elif path.params['status'] == 'no':
            await  websocket.send('server recv status:%s message:%s'%(path.params['status'], message))
        else:
            await  websocket.send('server recv invalid params')

def ws_server_open_sync(ip='localhost', port=8800):
    '''
    打开websocket server 监听 同步
    router 是路由
    '''
    asyncio.run(ws_server_main(ip, port))

def ws_server_open_async(ip='localhost', port=8800):
    '''
    打开websocket server 监听 异步
    router 是路由
    '''
    thread = threading.Thread(target=ws_server_open_sync, kwargs={'ip':ip, 'port':port})
    thread.start()
    return thread

#endregion



#region 客户端

async def ws_client_test(ip='localhost', port=8800, path='v1/test/ok', data='也许世界就这样'):
    '''
    ws 客户端测试
    # await ws_client_test()
    '''
    try:
        async with websockets.connect('ws://%s:%s/%s'%(ip, port, path)) as websocket:
            await websocket.send(data)
            recv_msg = await websocket.recv()
            zq_log.log_debug(recv_msg)
    except websockets.exceptions.ConnectionClosedError as e:
        zq_log.log_debug('connection closed error')
    except Exception as e:
        zq_log.log_error(e)

#endregion


import zq_log
zq_log.log_debug('好像没有维护了, 3.8上最新的pyttsx32.9用不了，这里需要指定安装 pip3 install pyttsx3==2.71')
import pyttsx3
def tts_pyttsx3_req(save, text):
    engine = pyttsx3.init() # object creation
    """ 把语音存储到文件 """
    engine.save_to_file(text,save)
    engine.runAndWait()
    engine.stop()

def tts_pyttsx3_speek(text):
    """更改速率"""
    engine = pyttsx3.init() # object creation
    rate = engine.getProperty('rate')   # getting details of current speaking rate
    engine.setProperty('rate', 125)     # setting up new voice rate


    """更改音量"""
    volume = engine.getProperty('volume')   #getting to know current volume level (min=0 and max=1)
    engine.setProperty('volume',1.0)    # setting up volume level  between 0 and 1

    """更改声音"""
    voices = engine.getProperty('voices')       #getting details of current voice
    engine.setProperty('voice', voices[0].id)  #changing index, changes voices. o for male  中文
    engine.say(text)
    engine.setProperty('voice', voices[1].id)   #changing index, changes voices. 1 for female  英文
    engine.say(text)
    engine.runAndWait()
    engine.stop()



import zq_selenium, time, zq_http
from selenium.webdriver.common.by import By
class tts_youdao_web_class:
    def __init__(self):
        self.selenium_tools = zq_selenium.selenium_tools_class()
        self.selenium_tools.web_open_url('https://ai.youdao.com/product-tts.s')
        self.selenium_tools.web_wait_node_element_click(By.ID, 'customSelectBtnText')
        time.sleep(1)
        self.selenium_tools.web_node_find_elements_click(By.CLASS_NAME, 'anchorColor', 2)

    def tts_youdao_req(self, save, text):
            self.selenium_tools.web_wait_node_element_click(By.ID, 'clearInput')
            self.selenium_tools.web_wait_node_element_send_keys(By.ID, 'inputText', text)
            self.selenium_tools.web_wait_node_element_click(By.ID, 'syntheticBtn')
            audio_src = self.selenium_tools.web_wait_node_get_attribute(By.ID, 'jp_audio_0', 'src')
            if zq_http.http_download(audio_src, save):
                print('下载语音成功:%s'%save)
            else:
                print('下载语音洗白:%s'%save)